@@ -1303,9 +1303,9 @@ int compile_to_object_file_cpp(const std::string &infile,
         }
         options += " -I" + rtlib_header_dir;
         std::string cmd = CXX + " " + options + " -o " + outfile + " -c " + cppfile;
-	if (verbose) {
+        if (verbose) {
             std::cout << cmd << std::endl;
-	}
+        }
         int err = system(cmd.c_str());
         if (err) {
             std::cout << "The command '" + cmd + "' failed." << std::endl;
@@ -1406,9 +1406,9 @@ int compile_to_object_file_c(const std::string &infile,
         std::string CXX = "gcc";
         std::string options = " -I" + rtlib_header_dir;
         std::string cmd = CXX + " " + options + " -o " + outfile + " -c " + cfile;
-	if (verbose) {
+        if (verbose) {
             std::cout << cmd << std::endl;
-	}
+        }
         int err = system(cmd.c_str());
         if (err) {
             std::cout << "The command '" + cmd + "' failed." << std::endl;
@@ -1560,6 +1560,21 @@ int link_executable(const std::vector<std::string> &infiles,
             }
             compile_cmd += runtime_library_dir + "\\lfortran_runtime_static.lib";
             run_cmd = outfile;
+        } else if (LCompilers::startswith(t, "wasm")) {
+            char* wasi_sdk_path = std::getenv("WASI_SDK_PATH");
+            if (wasi_sdk_path == nullptr) {
+                std::cerr << "WASI_SDK_PATH must be defined to use llvm->wasm\n";
+                return 11;
+            }
+            std::string CC = std::string(wasi_sdk_path) + "/bin/clang";
+            std::string options = " --target=wasm32-wasi -nostartfiles -Wl,--entry=_start -Wl,-lwasi-emulated-process-clocks";
+            std::string runtime_lib = "lfortran_runtime_wasm.o";
+            compile_cmd = CC + options + " -o " + outfile + " ";
+            for (auto &s : infiles) {
+                compile_cmd += s + " ";
+            }
+            compile_cmd += runtime_library_dir + "/" + runtime_lib;
+            compile_cmd +=  extra_linker_flags;
         } else {
             std::string CC;
             std::string base_path = "\"" + runtime_library_dir + "\"";
@@ -1593,9 +1608,9 @@ int link_executable(const std::vector<std::string> &infiles,
             for (auto &s : infiles) {
                 compile_cmd += s + " ";
             }
-	    if(!extra_library_flags.empty()) {
-	        compile_cmd += extra_library_flags + " ";
-	    }
+            if(!extra_library_flags.empty()) {
+                compile_cmd += extra_library_flags + " ";
+            }
             compile_cmd += + " -L"
                 + base_path + " -Wl,-rpath," + base_path;
             if (!extra_linker_flags.empty()) {
@@ -1646,18 +1661,18 @@ int link_executable(const std::vector<std::string> &infiles,
         for (auto &s : infiles) {
             cmd += s + " ";
         }
-	if(!extra_library_flags.empty()) {
-	    cmd += extra_library_flags + " ";
-	}
+        if(!extra_library_flags.empty()) {
+            cmd += extra_library_flags + " ";
+        }
         cmd += " -L" + base_path
             + " -Wl,-rpath," + base_path;
         if (!extra_linker_flags.empty()) {
             cmd += extra_linker_flags;
         }
         cmd += " -l" + runtime_lib + " -lm";
-	if (verbose) {
+        if (verbose) {
             std::cout << cmd << std::endl;
-	}
+        }
         int err = system(cmd.c_str());
         if (err) {
             std::cout << "The command '" + cmd + "' failed." << std::endl;
@@ -1684,13 +1699,13 @@ int link_executable(const std::vector<std::string> &infiles,
         for (auto &s : infiles) {
             cmd += s + " ";
         }
-	if(!extra_library_flags.empty()) {
-	    cmd += extra_library_flags + " ";
-	}
+        if(!extra_library_flags.empty()) {
+            cmd += extra_library_flags + " ";
+        }
         cmd += " " + post_options + " -lm";
-	if (verbose) {
+        if (verbose) {
             std::cout << cmd << std::endl;
-	}
+        }
         int err = system(cmd.c_str());
         if (err) {
             std::cout << "The command '" + cmd + "' failed." << std::endl;
@@ -1712,15 +1727,15 @@ int link_executable(const std::vector<std::string> &infiles,
         for (auto &s : infiles) {
             cmd += s + " ";
         }
-	if(!extra_library_flags.empty()) {
-	    cmd += extra_library_flags + " ";
-	}
+        if(!extra_library_flags.empty()) {
+            cmd += extra_library_flags + " ";
+        }
         cmd += " -L" + base_path
             + " -Wl,-rpath," + base_path;
         cmd += " -l" + runtime_lib + " -lm";
-	if (verbose) {
+        if (verbose) {
             std::cout << cmd << std::endl;
-	}
+        }
         int err = system(cmd.c_str());
         if (err) {
             std::cout << "The command '" + cmd + "' failed." << std::endl;
@@ -1741,6 +1756,8 @@ int link_executable(const std::vector<std::string> &infiles,
         run_cmd = "node --experimental-wasi-unstable-preview1 " + outfile + ".js";
     } else if (t == "x86_64-pc-windows-msvc") {
         run_cmd = outfile;
+    } else if (LCompilers::startswith(t, "wasm")) {
+        run_cmd = "wasmtime " + outfile + " --dir=.";
     } else {
         run_cmd = "./" + outfile;
     }
@@ -1971,7 +1988,7 @@ int main_app(int argc, char *argv[]) {
     app.add_option("files", arg_files, "Source files");
     app.add_flag("-S", arg_S, "Emit assembly, do not assemble or link");
     app.add_flag("-c", arg_c, "Compile and assemble, do not link");
-    app.add_option("-o", compiler_options.arg_o, "Specify the file to place the output into");
+    app.add_option("-o", compiler_options.arg_o, "Specify the file to place the compiler's output into");
     app.add_flag("-v", arg_v, "Be more verbose");
     app.add_flag("-E", arg_E, "Preprocess only; do not compile, assemble or link");
     app.add_option("-l", arg_l, "Link library option")->allow_extra_args(false);
