@@ -438,7 +438,7 @@ ast_t* data_implied_do(Allocator &al, Location &loc,
 
 static inline var_sym_t* VARSYM(Allocator &al, Location &l,
         char* name, dimension_t* dim, size_t n_dim,
-        codimension_t* codim, size_t n_codim, expr_t* init,
+        codimension_t* codim, size_t n_codim, expr_t* length, expr_t* init,
         LCompilers::LFortran::AST::symbolType sym, decl_attribute_t* x)
 {
     var_sym_t *r = al.allocate<var_sym_t>(1);
@@ -448,6 +448,7 @@ static inline var_sym_t* VARSYM(Allocator &al, Location &l,
     r->n_dim = n_dim;
     r->m_codim = codim;
     r->n_codim = n_codim;
+    r->m_length = length;
     r->m_initializer = init;
     r->m_sym = sym;
     r->m_spec = x;
@@ -455,21 +456,27 @@ static inline var_sym_t* VARSYM(Allocator &al, Location &l,
 }
 
 #define VAR_SYM_NAME(name, sym, loc) VARSYM(p.m_a, loc, \
-        name2char(name), nullptr, 0, nullptr, 0, nullptr, sym, nullptr)
+        name2char(name), nullptr, 0, nullptr, 0, nullptr, nullptr, sym, nullptr)
 #define VAR_SYM_DIM_EXPR(exp, sym, loc) VARSYM(p.m_a, loc, nullptr, \
-        nullptr, 0, nullptr, 0, down_cast<expr_t>(exp), sym, nullptr)
+        nullptr, 0, nullptr, 0, nullptr, down_cast<expr_t>(exp), sym, nullptr)
 #define VAR_SYM_DIM_INIT(name, dim, n_dim, init, sym, loc) VARSYM(p.m_a, loc, \
-        name2char(name), dim, n_dim, nullptr, 0, \
+        name2char(name), dim, n_dim, nullptr, 0, nullptr, \
+        down_cast<expr_t>(init), sym, nullptr)
+#define VAR_SYM_DIM_LEN(name, dim, n_dim, len, sym, loc) VARSYM(p.m_a, loc, \
+        name2char(name), dim, n_dim, nullptr, 0, down_cast<expr_t>(len), \
+        nullptr, sym, nullptr)
+#define VAR_SYM_DIM_LEN_INIT(name, dim, n_dim, len, init, sym, loc) VARSYM(p.m_a, loc, \
+        name2char(name), dim, n_dim, nullptr, 0, down_cast<expr_t>(len), \
         down_cast<expr_t>(init), sym, nullptr)
 #define VAR_SYM_DIM(name, dim, n_dim, sym, loc) VARSYM(p.m_a, loc, \
-        name2char(name), dim, n_dim, nullptr, 0, nullptr, sym, nullptr)
+        name2char(name), dim, n_dim, nullptr, 0, nullptr, nullptr, sym, nullptr)
 #define VAR_SYM_CODIM(name, codim, n_codim, sym, loc) VARSYM(p.m_a, loc, \
-        name2char(name), nullptr, 0, codim, n_codim, nullptr, sym, nullptr)
+        name2char(name), nullptr, 0, codim, n_codim, nullptr, nullptr, sym, nullptr)
 #define VAR_SYM_DIM_CODIM(name, dim, n_dim, codim, n_codim, sym, loc) \
         VARSYM(p.m_a, loc, name2char(name), \
-        dim, n_dim, codim, n_codim, nullptr, sym, nullptr)
+        dim, n_dim, codim, n_codim, nullptr, nullptr, sym, nullptr)
 #define VAR_SYM_SPEC(x, sym, loc) VARSYM(p.m_a, loc, \
-        nullptr, nullptr, 0, nullptr, 0, nullptr, sym, \
+        nullptr, nullptr, 0, nullptr, 0, nullptr, nullptr, sym, \
         down_cast<decl_attribute_t>(x))
 #define DECL_ASSIGNMENT(l) make_AttrAssignment_t(p.m_a, l)
 #define DECL_OP(op, l) make_AttrIntrinsicOperator_t(p.m_a, l, op)
