@@ -403,6 +403,10 @@ class ASRBuilder {
         return EXPR(ASR::make_RealBinOp_t(al, loc, left, ASR::binopType::Mul, right, t, nullptr));
     }
 
+    inline ASR::expr_t* r_tDiv(ASR::expr_t* left, ASR::expr_t* right, ASR::ttype_t* t) {
+        return EXPR(ASR::make_RealBinOp_t(al, loc, left, ASR::binopType::Div, right, t, nullptr));
+    }
+
     inline ASR::expr_t* iPow(ASR::expr_t* left, ASR::expr_t* right, ASR::ttype_t* t) {
         return EXPR(ASR::make_IntegerBinOp_t(al, loc, left, ASR::binopType::Pow, right, t, nullptr));
     }
@@ -646,7 +650,7 @@ class ASRBuilder {
         for (auto &x: body) m_body.push_back(al, x);
 
         return STMT(ASR::make_WhileLoop_t(al, loc, nullptr, a_test,
-            m_body.p, m_body.n));
+            m_body.p, m_body.n, nullptr, 0));
     }
 
     ASR::stmt_t *Exit(char* loop_name) {
@@ -799,7 +803,8 @@ class ASRBuilder {
     ASR::expr_t* Call(ASR::symbol_t* s, Vec<ASR::call_arg_t>& args,
                       ASR::ttype_t* return_type) {
         return ASRUtils::EXPR(ASRUtils::make_FunctionCall_t_util(al, loc,
-                s, s, args.p, args.size(), return_type, nullptr, nullptr));
+                s, s, args.p, args.size(), return_type, nullptr, nullptr,
+                false));
     }
 
     ASR::expr_t* Call(ASR::symbol_t* s, Vec<ASR::expr_t *>& args,
@@ -807,13 +812,15 @@ class ASRBuilder {
         Vec<ASR::call_arg_t> args_; args_.reserve(al, 2);
         visit_expr_list(al, args, args_);
         return ASRUtils::EXPR(ASRUtils::make_FunctionCall_t_util(al, loc,
-                s, s, args_.p, args_.size(), return_type, nullptr, nullptr));
+                s, s, args_.p, args_.size(), return_type, nullptr, nullptr,
+                false));
     }
 
     ASR::expr_t* Call(ASR::symbol_t* s, Vec<ASR::call_arg_t>& args,
                       ASR::ttype_t* return_type, ASR::expr_t* value) {
         return ASRUtils::EXPR(ASRUtils::make_FunctionCall_t_util(al, loc,
-                s, s, args.p, args.size(), return_type, value, nullptr));
+                s, s, args.p, args.size(), return_type, value, nullptr,
+                false));
     }
 
     ASR::stmt_t* SubroutineCall(ASR::symbol_t* s, Vec<ASR::call_arg_t>& args) {
@@ -911,7 +918,7 @@ class ASRBuilder {
         head.m_increment = step;
         Vec<ASR::stmt_t *> body;
         body.from_pointer_n_copy(al, &loop_body[0], loop_body.size());
-        return STMT(ASR::make_DoLoop_t(al, loc, nullptr, head, body.p, body.n));
+        return STMT(ASR::make_DoLoop_t(al, loc, nullptr, head, body.p, body.n, nullptr, 0));
     }
 
     template <typename LOOP_BODY>
@@ -937,7 +944,7 @@ class ASRBuilder {
                 doloop_body.push_back(al, doloop);
             }
             doloop = ASRUtils::STMT(ASR::make_DoLoop_t(al, loc, nullptr,
-                        head, doloop_body.p, doloop_body.size()));
+                        head, doloop_body.p, doloop_body.size(), nullptr, 0));
         }
         return doloop;
     }
@@ -964,7 +971,7 @@ class ASRBuilder {
                 doloop_body.push_back(al, doloop);
             }
             doloop = ASRUtils::STMT(ASR::make_DoLoop_t(al, loc, nullptr,
-                        head, doloop_body.p, doloop_body.size()));
+                        head, doloop_body.p, doloop_body.size(), nullptr, 0));
         }
         return doloop;
     }
@@ -1040,6 +1047,23 @@ class ASRBuilder {
             nullptr, nullptr));
     }
 
+    ASR::symbol_t* create_c_func(std::string c_func_name, SymbolTable* fn_symtab, ASR::ttype_t* return_type, int n_args, Vec<ASR::ttype_t*>& arg_types) {
+        SymbolTable *fn_symtab_1 = al.make_new<SymbolTable>(fn_symtab);
+        Vec<ASR::expr_t*> args_1; args_1.reserve(al, n_args);
+        for (int i = 0; i < n_args; i++) {
+            args_1.push_back(al, this->Variable(fn_symtab_1, "x_"+std::to_string(i), arg_types[i],
+                ASR::intentType::In, ASR::abiType::BindC, true));
+        }
+        ASR::expr_t *return_var_1 = this->Variable(fn_symtab_1, c_func_name,
+            return_type, ASRUtils::intent_return_var, ASR::abiType::BindC, false);
+
+        SetChar dep_1; dep_1.reserve(al, 1);
+        Vec<ASR::stmt_t*> body_1; body_1.reserve(al, 1);
+        ASR::symbol_t *s = make_ASR_Function_t(c_func_name, fn_symtab_1, dep_1, args_1,
+            body_1, return_var_1, ASR::abiType::BindC, ASR::deftypeType::Interface, s2c(al, c_func_name));
+        return s;
+    }
+
 };
 
 } // namespace LCompilers::ASRUtils
