@@ -124,6 +124,20 @@ public:
         current_symtab = nullptr;
     }
 
+    void visit_Select(const Select_t& x) {
+        bool fall_through = false;
+        for( size_t i = 0; i < x.n_body; i++ ) {
+            if( ASR::is_a<ASR::CaseStmt_t>(*x.m_body[i]) ) {
+                ASR::CaseStmt_t* case_stmt_t = ASR::down_cast<ASR::CaseStmt_t>(x.m_body[i]);
+                fall_through = fall_through || case_stmt_t->m_fall_through;
+            }
+        }
+        require(fall_through == x.m_enable_fall_through,
+            "Select_t::m_enable_fall_through should be " +
+            std::to_string(x.m_enable_fall_through));
+        BaseWalkVisitor<VerifyVisitor>::visit_Select(x);
+    }
+
     // --------------------------------------------------------
     // symbol instances:
 
@@ -347,13 +361,15 @@ public:
         ASR::expr_t* target = x.m_target;
         if( ASR::is_a<ASR::Var_t>(*target) ) {
             ASR::Var_t* target_Var = ASR::down_cast<ASR::Var_t>(target);
+            bool is_target_const = false;
             ASR::ttype_t* target_type = nullptr;
-            if( ASR::is_a<ASR::Variable_t>(*target_Var->m_v) ||
-                (ASR::is_a<ASR::ExternalSymbol_t>(*target_Var->m_v) &&
-                 ASR::down_cast<ASR::ExternalSymbol_t>(target_Var->m_v)->m_external) ) {
-                target_type = ASRUtils::expr_type(target);
+            ASR::symbol_t* target_sym = ASRUtils::symbol_get_past_external(target_Var->m_v);
+            if( target_sym && ASR::is_a<ASR::Variable_t>(*target_sym) ) {
+                ASR::Variable_t* var = ASR::down_cast<ASR::Variable_t>(target_sym);
+                target_type = var->m_type;
+                is_target_const = var->m_storage == ASR::storage_typeType::Parameter;
             }
-            if( target_type && ASR::is_a<ASR::Const_t>(*target_type) ) {
+            if( is_target_const ) {
                 std::string variable_name = ASRUtils::symbol_name(target_Var->m_v);
                 require(const_assigned.find(std::make_pair(current_symtab->counter,
                     variable_name)) == const_assigned.end(),
@@ -511,6 +527,8 @@ public:
                 ASR::is_a<ASR::CustomOperator_t>(*a.second) ) {
                 continue ;
             }
+            // TODO: Uncomment the following line
+            // ASR::ttype_t* var_type = ASRUtils::extract_type(ASRUtils::symbol_type(a.second));
             ASR::ttype_t* var_type = ASRUtils::type_get_past_pointer(ASRUtils::symbol_type(a.second));
             char* aggregate_type_name = nullptr;
             ASR::symbol_t* sym = nullptr;
@@ -698,6 +716,7 @@ public:
             ASR::Module_t *m = ASRUtils::get_sym_module(x.m_external);
             ASR::StructType_t* sm = nullptr;
             ASR::EnumType_t* em = nullptr;
+            ASR::UnionType_t* um = nullptr;
             ASR::Function_t* fm = nullptr;
             bool is_valid_owner = false;
             is_valid_owner = m != nullptr && ((ASR::symbol_t*) m == ASRUtils::get_asr_owner(x.m_external));
@@ -706,13 +725,17 @@ public:
                 ASR::symbol_t* asr_owner_sym = ASRUtils::get_asr_owner(x.m_external);
                 is_valid_owner = (ASR::is_a<ASR::StructType_t>(*asr_owner_sym) ||
                                   ASR::is_a<ASR::EnumType_t>(*asr_owner_sym) ||
-                                  ASR::is_a<ASR::Function_t>(*asr_owner_sym));
+                                  ASR::is_a<ASR::Function_t>(*asr_owner_sym) ||
+                                  ASR::is_a<ASR::UnionType_t>(*asr_owner_sym));
                 if( ASR::is_a<ASR::StructType_t>(*asr_owner_sym) ) {
                     sm = ASR::down_cast<ASR::StructType_t>(asr_owner_sym);
                     asr_owner_name = sm->m_name;
                 } else if( ASR::is_a<ASR::EnumType_t>(*asr_owner_sym) ) {
                     em = ASR::down_cast<ASR::EnumType_t>(asr_owner_sym);
                     asr_owner_name = em->m_name;
+                } else if( ASR::is_a<ASR::UnionType_t>(*asr_owner_sym) ) {
+                    um = ASR::down_cast<ASR::UnionType_t>(asr_owner_sym);
+                    asr_owner_name = um->m_name;
                 } else if( ASR::is_a<ASR::Function_t>(*asr_owner_sym) ) {
                     fm = ASR::down_cast<ASR::Function_t>(asr_owner_sym);
                     asr_owner_name = fm->m_name;
@@ -741,6 +764,8 @@ public:
                 s = em->m_symtab->resolve_symbol(std::string(x.m_original_name));
             } else if( fm ) {
                 s = fm->m_symtab->resolve_symbol(std::string(x.m_original_name));
+            } else if( um ) {
+                s = um->m_symtab->resolve_symbol(std::string(x.m_original_name));
             }
             require(s != nullptr,
                 "ExternalSymbol::m_original_name ('"
@@ -1128,13 +1153,9 @@ public:
         for (size_t i = 0; i < x.n_args; i++) {
             require(!ASR::is_a<ASR::ArrayConstant_t>(*x.m_args[i]),
                 "ArrayConstant cannot have ArrayConstant as its elements");
+            ASR::expr_t* arg_value = ASRUtils::expr_value(x.m_args[i]);
             require(
-                ASR::is_a<ASR::IntegerConstant_t>(*x.m_args[i]) ||
-                ASR::is_a<ASR::RealConstant_t>(*x.m_args[i]) ||
-                ASR::is_a<ASR::ComplexConstant_t>(*x.m_args[i]) ||
-                ASR::is_a<ASR::LogicalConstant_t>(*x.m_args[i]) ||
-                ASR::is_a<ASR::StringConstant_t>(*x.m_args[i]) ||
-                ASR::is_a<ASR::UnsignedIntegerConstant_t>(*x.m_args[i]),
+                ASRUtils::is_value_constant(arg_value),
                 "ArrayConstant must have constant values");
         }
 
@@ -1144,14 +1165,14 @@ public:
     void visit_dimension(const dimension_t &x) {
         if (x.m_start) {
             require_with_loc(ASRUtils::is_integer(
-                *ASRUtils::type_get_past_const(ASRUtils::expr_type(x.m_start))),
+                *ASRUtils::expr_type(x.m_start)),
                 "Start dimension must be a signed integer", x.loc);
             visit_expr(*x.m_start);
         }
 
         if (x.m_length) {
             require_with_loc(ASRUtils::is_integer(
-                *ASRUtils::type_get_past_const(ASRUtils::expr_type(x.m_length))),
+                *ASRUtils::expr_type(x.m_length)),
                 "Length dimension must be a signed integer", x.loc);
             visit_expr(*x.m_length);
         }
