@@ -38,12 +38,36 @@ intrinsic_funcs_args = {
             "ret_type_arg_idx": 0
         },
     ],
+    "Modulo": [
+        {
+            "args": [("int", "int"), ("real", "real")],
+            "ret_type_arg_idx": 0
+        },
+    ],
     "BesselJ0": [
         {
             "args": [("real",)],
             "ret_type_arg_idx": 0
         },
     ],
+    "BesselJ1": [
+        {
+            "args": [("real",)],
+            "ret_type_arg_idx": 0
+        },
+    ],
+    "BesselY0": [
+        {
+            "args": [("real",)],
+            "ret_type_arg_idx": 0
+        },
+    ],
+    "BesselY1": [
+        {
+            "args": [("real",)],
+            "ret_type_arg_idx": 0
+        },
+    ],
     "Mvbits": [
         {
             "args": [("int", "int", "int", "int", "int")],
@@ -205,6 +229,18 @@ intrinsic_funcs_args = {
             "kind_arg": True
         }
     ],
+    "Asind": [
+        {
+            "args": [("real",)],
+            "ret_type_arg_idx": 0
+        }
+    ],
+    "Acosd": [
+        {
+            "args": [("real",)],
+            "ret_type_arg_idx": 0
+        }
+    ],
     "Sqrt": [
         {
             "args": [("real",), ("complex",)],
@@ -368,6 +404,12 @@ intrinsic_funcs_args = {
             "return": "int32"
         }
     ],
+    "BitSize": [
+        {
+            "args": [("int",)],
+            "return": "int32"
+        }
+    ],
     "Range": [
         {
             "args": [("int",), ("real",), ("complex",)],
@@ -510,6 +552,7 @@ compile_time_only_fn = [
     "Rank",
     "Tiny",
     "Huge",
+    "BitSize",
 ]
 
 type_to_asr_type_check = {
@@ -539,7 +582,7 @@ indent = "    "
 def compute_arg_types(indent, no_of_args, args_arr):
     global src
     for i in range(no_of_args):
-        src += indent + f"ASR::ttype_t *arg_type{i} = ASRUtils::type_get_past_const(ASRUtils::expr_type({args_arr}[{i}]));\n"
+        src += indent + f"ASR::ttype_t *arg_type{i} = ASRUtils::expr_type({args_arr}[{i}]);\n"
 
 def compute_arg_condition(no_of_args, args_lists):
     condition = []
@@ -660,7 +703,10 @@ def add_create_func_return_src(func_name):
             src += indent * 3 + f"args_values.push_back(al, expr_value(m_args[{_i}]));\n"
         src += indent * 3 +     f"m_value = eval_{func_name}(al, loc, return_type, args_values, diag);\n"
         src += indent * 2 + "}\n"
-        src += indent * 2 + f"return ASR::make_IntrinsicElementalFunction_t(al, loc, static_cast<int64_t>(IntrinsicElementalFunctions::{func_name}), m_args.p, m_args.n, 0, return_type, m_value);\n"
+        if "null" in intrinsic_funcs_ret_type.get(func_name, []):
+            src += indent * 2 + f"return ASR::make_Expr_t(al, loc, ASRUtils::EXPR(ASR::make_IntrinsicElementalFunction_t(al, loc, static_cast<int64_t>(IntrinsicElementalFunctions::{func_name}), m_args.p, m_args.n, 0, return_type, m_value)));\n"
+        else:
+            src += indent * 2 + f"return ASR::make_IntrinsicElementalFunction_t(al, loc, static_cast<int64_t>(IntrinsicElementalFunctions::{func_name}), m_args.p, m_args.n, 0, return_type, m_value);\n"
 
 def gen_verify_args(func_name):
     global src
