@@ -18,12 +18,15 @@ namespace ASRUtils {
 
 inline std::string get_intrinsic_name(int x) {
     switch (x) {
+        INTRINSIC_NAME_CASE(ObjectType)
         INTRINSIC_NAME_CASE(Kind)
         INTRINSIC_NAME_CASE(Rank)
         INTRINSIC_NAME_CASE(Sin)
         INTRINSIC_NAME_CASE(Cos)
         INTRINSIC_NAME_CASE(Tan)
         INTRINSIC_NAME_CASE(Asin)
+        INTRINSIC_NAME_CASE(Asind)
+        INTRINSIC_NAME_CASE(Acosd)
         INTRINSIC_NAME_CASE(Acos)
         INTRINSIC_NAME_CASE(Atan)
         INTRINSIC_NAME_CASE(Sinh)
@@ -51,7 +54,11 @@ inline std::string get_intrinsic_name(int x) {
         INTRINSIC_NAME_CASE(FloorDiv)
         INTRINSIC_NAME_CASE(Mod)
         INTRINSIC_NAME_CASE(Trailz)
+        INTRINSIC_NAME_CASE(Modulo)
         INTRINSIC_NAME_CASE(BesselJ0)
+        INTRINSIC_NAME_CASE(BesselJ1)
+        INTRINSIC_NAME_CASE(BesselY0)
+        INTRINSIC_NAME_CASE(BesselY1)
         INTRINSIC_NAME_CASE(Mvbits)
         INTRINSIC_NAME_CASE(Shiftr)
         INTRINSIC_NAME_CASE(Rshift)
@@ -128,6 +135,7 @@ inline std::string get_intrinsic_name(int x) {
         INTRINSIC_NAME_CASE(Epsilon)
         INTRINSIC_NAME_CASE(Precision)
         INTRINSIC_NAME_CASE(Tiny)
+        INTRINSIC_NAME_CASE(BitSize)
         INTRINSIC_NAME_CASE(Conjg)
         INTRINSIC_NAME_CASE(Huge)
         INTRINSIC_NAME_CASE(Dprod)
@@ -165,6 +173,8 @@ namespace IntrinsicElementalFunctionRegistry {
     static const std::map<int64_t,
         std::tuple<impl_function,
                    verify_function>>& intrinsic_function_by_id_db = {
+        {static_cast<int64_t>(IntrinsicElementalFunctions::ObjectType),
+            {nullptr, &ObjectType::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Gamma),
             {&Gamma::instantiate_Gamma, &UnaryIntrinsicFunction::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Log10),
@@ -221,10 +231,22 @@ namespace IntrinsicElementalFunctionRegistry {
             {&FloorDiv::instantiate_FloorDiv, &FloorDiv::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Mod),
             {&Mod::instantiate_Mod, &Mod::verify_args}},
+        {static_cast<int64_t>(IntrinsicElementalFunctions::Asind),
+            {&Asind::instantiate_Asind, &Asind::verify_args}},
+        {static_cast<int64_t>(IntrinsicElementalFunctions::Acosd),
+            {&Acosd::instantiate_Acosd, &Acosd::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Trailz),
             {&Trailz::instantiate_Trailz, &Trailz::verify_args}},
+        {static_cast<int64_t>(IntrinsicElementalFunctions::Modulo),
+            {&Modulo::instantiate_Modulo, &Modulo::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::BesselJ0),
             {&BesselJ0::instantiate_BesselJ0, &BesselJ0::verify_args}},
+        {static_cast<int64_t>(IntrinsicElementalFunctions::BesselJ1),
+            {&BesselJ1::instantiate_BesselJ1, &BesselJ1::verify_args}},
+        {static_cast<int64_t>(IntrinsicElementalFunctions::BesselY0),
+            {&BesselY0::instantiate_BesselY0, &BesselY0::verify_args}},
+        {static_cast<int64_t>(IntrinsicElementalFunctions::BesselY1),
+            {&BesselY1::instantiate_BesselY1, &BesselY1::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Mvbits),
             {&Mvbits::instantiate_Mvbits, &Mvbits::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Shiftr),
@@ -385,6 +407,8 @@ namespace IntrinsicElementalFunctionRegistry {
             {nullptr, &Precision::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Tiny),
             {nullptr, &UnaryIntrinsicFunction::verify_args}},
+        {static_cast<int64_t>(IntrinsicElementalFunctions::BitSize),
+            {nullptr, &UnaryIntrinsicFunction::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Huge),
             {nullptr, &UnaryIntrinsicFunction::verify_args}},
         {static_cast<int64_t>(IntrinsicElementalFunctions::SelectedIntKind),
@@ -442,6 +466,8 @@ namespace IntrinsicElementalFunctionRegistry {
     };
 
     static const std::map<int64_t, std::string>& intrinsic_function_id_to_name = {
+        {static_cast<int64_t>(IntrinsicElementalFunctions::ObjectType),
+            "type"},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Gamma),
             "gamma"},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Log),
@@ -500,10 +526,20 @@ namespace IntrinsicElementalFunctionRegistry {
             "floordiv"},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Mod),
             "mod"},
+        {static_cast<int64_t>(IntrinsicElementalFunctions::Asind),
+            "asind"},
+        {static_cast<int64_t>(IntrinsicElementalFunctions::Acosd),
+            "acosd"},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Trailz),
             "trailz"},
+        {static_cast<int64_t>(IntrinsicElementalFunctions::Modulo),
+            "modulo"},
         {static_cast<int64_t>(IntrinsicElementalFunctions::BesselJ0),
             "bessel_j0"},
+        {static_cast<int64_t>(IntrinsicElementalFunctions::BesselY0),
+            "bessel_y0"},
+        {static_cast<int64_t>(IntrinsicElementalFunctions::BesselY1),
+            "bessel_y1"},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Mvbits),
             "mvbits"},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Shiftr),
@@ -666,6 +702,8 @@ namespace IntrinsicElementalFunctionRegistry {
             "precision"},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Tiny),
             "tiny"},
+        {static_cast<int64_t>(IntrinsicElementalFunctions::BitSize),
+            "bit_size"},
         {static_cast<int64_t>(IntrinsicElementalFunctions::Huge),
             "huge"},
         {static_cast<int64_t>(IntrinsicElementalFunctions::SymbolicSymbol),
@@ -720,6 +758,7 @@ namespace IntrinsicElementalFunctionRegistry {
     static const std::map<std::string,
         std::tuple<create_intrinsic_function,
                     eval_intrinsic_function>>& intrinsic_function_by_name_db = {
+                {"type", {&ObjectType::create_ObjectType, &ObjectType::eval_ObjectType}},
                 {"gamma", {&Gamma::create_Gamma, &Gamma::eval_Gamma}},
                 {"log", {&Log::create_Log, &Log::eval_Log}},
                 {"log10", {&Log10::create_Log10, &Log10::eval_Log10}},
@@ -749,8 +788,14 @@ namespace IntrinsicElementalFunctionRegistry {
                 {"fma", {&FMA::create_FMA, &FMA::eval_FMA}},
                 {"floordiv", {&FloorDiv::create_FloorDiv, &FloorDiv::eval_FloorDiv}},
                 {"mod", {&Mod::create_Mod, &Mod::eval_Mod}},
+                {"asind", {&Asind::create_Asind, &Asind::eval_Asind}},
+                {"acosd", {&Acosd::create_Acosd, &Acosd::eval_Acosd}},
                 {"trailz", {&Trailz::create_Trailz, &Trailz::eval_Trailz}},
+                {"modulo", {&Modulo::create_Modulo, &Modulo::eval_Modulo}},
                 {"bessel_j0", {&BesselJ0::create_BesselJ0, &BesselJ0::eval_BesselJ0}},
+                {"bessel_j1", {&BesselJ1::create_BesselJ1, &BesselJ1::eval_BesselJ1}},
+                {"bessel_y0", {&BesselY0::create_BesselY0, &BesselY0::eval_BesselY0}},
+                {"bessel_y1", {&BesselY1::create_BesselY1, &BesselY1::eval_BesselY1}},
                 {"mvbits", {&Mvbits::create_Mvbits, &Mvbits::eval_Mvbits}},
                 {"shiftr", {&Shiftr::create_Shiftr, &Shiftr::eval_Shiftr}},
                 {"rshift", {&Rshift::create_Rshift, &Rshift::eval_Rshift}},
@@ -832,6 +877,7 @@ namespace IntrinsicElementalFunctionRegistry {
                 {"epsilon", {&Epsilon::create_Epsilon, &Epsilon::eval_Epsilon}},
                 {"precision", {&Precision::create_Precision, &Precision::eval_Precision}},
                 {"tiny", {&Tiny::create_Tiny, &Tiny::eval_Tiny}},
+                {"bit_size", {&BitSize::create_BitSize, &BitSize::eval_BitSize}},
                 {"conjg", {&Conjg::create_Conjg, &Conjg::eval_Conjg}},
                 {"huge", {&Huge::create_Huge, &Huge::eval_Huge}},
                 {"Symbol", {&SymbolicSymbol::create_SymbolicSymbol, &SymbolicSymbol::eval_SymbolicSymbol}},
