@@ -10,6 +10,7 @@
 #include <limits.h>
 #include <ctype.h>
 
+#define PI 3.14159265358979323846
 #if defined(_WIN32)
 #  include <winsock2.h>
 #  include <io.h>
@@ -23,6 +24,11 @@
 
 #ifdef HAVE_RUNTIME_STACKTRACE
 
+#ifdef COMPILE_TO_WASM
+    #undef HAVE_LFORTRAN_MACHO
+    #undef HAVE_LFORTRAN_LINK
+#endif
+
 #ifdef HAVE_LFORTRAN_LINK
 // For dl_iterate_phdr() functionality
 #  include <link.h>
@@ -997,215 +1003,42 @@ LFORTRAN_API double _lfortran_dlog_gamma(double x)
 
 // besselj0 --------------------------------------------------------------------
 
-/**
-* Ported from implementation done at:
-* https://github.com/stdlib-js/stdlib/blob/develop/lib/node_modules/%40stdlib/math/base/special/besselj0/lib/main.js
-*
-* All credits to the original authors of the implementation.
-*/
-
-/**
-* Evaluates a rational function (i.e., the ratio of two polynomials described by the coefficients stored in \\(P\\) and \\(Q\\)).
-*
-* ## Notes
-*
-* -   Coefficients should be sorted in ascending degree.
-* -   The implementation uses [Horner's rule][horners-method] for efficient computation.
-*
-* [horners-method]: https://en.wikipedia.org/wiki/Horner%27s_method
-*
-* @param x    value at which to evaluate the rational function
-* @return     evaluated rational function
-*/
-static double rational_p1q1( const double x ) {
-	double ax;
-	double ix;
-	double s1;
-	double s2;
-	if ( x == 0.0 ) {
-		return -0.17291506903064494;
-	}
-	if ( x < 0.0 ) {
-		ax = -x;
-	} else {
-		ax = x;
-	}
-	if ( ax <= 1.0 ) {
-		s1 = -412986685009.9087 + (x * (27282507878.60594 + (x * (-621407004.2354012 + (x * (6630299.79048338 + (x * (-36629.81465510709 + (x * (103.44222815443189 + (x * -0.12117036164593528)))))))))));
-		s2 = 2388378799633.229 + (x * (26328198300.85965 + (x * (139850973.72263435 + (x * (456126.9622421994 + (x * (936.1402239233771 + (x * (1.0 + (x * 0.0)))))))))));
-	} else {
-		ix = 1.0 / x;
-		s1 = -0.12117036164593528 + (ix * (103.44222815443189 + (ix * (-36629.81465510709 + (ix * (6630299.79048338 + (ix * (-621407004.2354012 + (ix * (27282507878.60594 + (ix * -412986685009.9087)))))))))));
-		s2 = 0.0 + (ix * (1.0 + (ix * (936.1402239233771 + (ix * (456126.9622421994 + (ix * (139850973.72263435 + (ix * (26328198300.85965 + (ix * 2388378799633.229)))))))))));
-	}
-	return s1 / s2;
-}
-
-/**
-* Evaluates a rational function (i.e., the ratio of two polynomials described by the coefficients stored in \\(P\\) and \\(Q\\)).
-*
-* ## Notes
-*
-* -   Coefficients should be sorted in ascending degree.
-* -   The implementation uses [Horner's rule][horners-method] for efficient computation.
-*
-* [horners-method]: https://en.wikipedia.org/wiki/Horner%27s_method
-*
-* @param x    value at which to evaluate the rational function
-* @return     evaluated rational function
-*/
-static double rational_p2q2( const double x ) {
-	double ax;
-	double ix;
-	double s1;
-	double s2;
-	if ( x == 0.0 ) {
-		return 0.005119512965174424;
-	}
-	if ( x < 0.0 ) {
-		ax = -x;
-	} else {
-		ax = x;
-	}
-	if ( ax <= 1.0 ) {
-		s1 = -1831.9397969392085 + (x * (-12254.07816137899 + (x * (-7287.970246446462 + (x * (10341.910641583727 + (x * (11725.046279757104 + (x * (4417.670702532509 + (x * (743.2119668062425 + (x * 48.5917033559165)))))))))))));
-		s2 = -357834.78026152303 + (x * (245991.0226258631 + (x * (-84055.06259116957 + (x * (18680.99000835919 + (x * (-2945.876654550934 + (x * (333.07310774649073 + (x * (-25.258076240801554 + (x * 1.0)))))))))))));
-	} else {
-		ix = 1.0 / x;
-		s1 = 48.5917033559165 + (ix * (743.2119668062425 + (ix * (4417.670702532509 + (ix * (11725.046279757104 + (ix * (10341.910641583727 + (ix * (-7287.970246446462 + (ix * (-12254.07816137899 + (ix * -1831.9397969392085)))))))))))));
-		s2 = 1.0 + (ix * (-25.258076240801554 + (ix * (333.07310774649073 + (ix * (-2945.876654550934 + (ix * (18680.99000835919 + (ix * (-84055.06259116957 + (ix * (245991.0226258631 + (ix * -357834.78026152303)))))))))))));
-	}
-	return s1 / s2;
-}
-
-/**
-* Evaluates a rational function (i.e., the ratio of two polynomials described by the coefficients stored in \\(P\\) and \\(Q\\)).
-*
-* ## Notes
-*
-* -   Coefficients should be sorted in ascending degree.
-* -   The implementation uses [Horner's rule][horners-method] for efficient computation.
-*
-* [horners-method]: https://en.wikipedia.org/wiki/Horner%27s_method
-*
-* @param x    value at which to evaluate the rational function
-* @return     evaluated rational function
-*/
-static double rational_pcqc( const double x ) {
-	double ax;
-	double ix;
-	double s1;
-	double s2;
-	if ( x == 0.0 ) {
-		return 1.0;
-	}
-	if ( x < 0.0 ) {
-		ax = -x;
-	} else {
-		ax = x;
-	}
-	if ( ax <= 1.0 ) {
-		s1 = 22779.090197304686 + (x * (41345.38663958076 + (x * (21170.523380864943 + (x * (3480.648644324927 + (x * (153.76201909008356 + (x * 0.8896154842421046)))))))));
-		s2 = 22779.090197304686 + (x * (41370.41249551042 + (x * (21215.350561880117 + (x * (3502.8735138235606 + (x * (157.11159858080893 + (x * 1.0)))))))));
-	} else {
-		ix = 1.0 / x;
-		s1 = 0.8896154842421046 + (ix * (153.76201909008356 + (ix * (3480.648644324927 + (ix * (21170.523380864943 + (ix * (41345.38663958076 + (ix * 22779.090197304686)))))))));
-		s2 = 1.0 + (ix * (157.11159858080893 + (ix * (3502.8735138235606 + (ix * (21215.350561880117 + (ix * (41370.41249551042 + (ix * 22779.090197304686)))))))));
-	}
-	return s1 / s2;
-}
-
-/**
-* Evaluates a rational function (i.e., the ratio of two polynomials described by the coefficients stored in \\(P\\) and \\(Q\\)).
-*
-* ## Notes
-*
-* -   Coefficients should be sorted in ascending degree.
-* -   The implementation uses [Horner's rule][horners-method] for efficient computation.
-*
-* [horners-method]: https://en.wikipedia.org/wiki/Horner%27s_method
-*
-* @param x    value at which to evaluate the rational function
-* @return     evaluated rational function
-*/
-static double rational_psqs( const double x ) {
-	double ax;
-	double ix;
-	double s1;
-	double s2;
-	if ( x == 0.0 ) {
-		return -0.015625;
-	}
-	if ( x < 0.0 ) {
-		ax = -x;
-	} else {
-		ax = x;
-	}
-	if ( ax <= 1.0 ) {
-		s1 = -89.22660020080009 + (x * (-185.91953644342993 + (x * (-111.83429920482737 + (x * (-22.300261666214197 + (x * (-1.244102674583564 + (x * -0.008803330304868075)))))))));
-		s2 = 5710.502412851206 + (x * (11951.131543434614 + (x * (7264.278016921102 + (x * (1488.7231232283757 + (x * (90.59376959499312 + (x * 1.0)))))))));
-	} else {
-		ix = 1.0 / x;
-		s1 = -0.008803330304868075 + (ix * (-1.244102674583564 + (ix * (-22.300261666214197 + (ix * (-111.83429920482737 + (ix * (-185.91953644342993 + (ix * -89.22660020080009)))))))));
-		s2 = 1.0 + (ix * (90.59376959499312 + (ix * (1488.7231232283757 + (ix * (7264.278016921102 + (ix * (11951.131543434614 + (ix * 5710.502412851206)))))))));
-	}
-	return s1 / s2;
-}
-
 LFORTRAN_API double _lfortran_dbesselj0( double x ) {
-	double rc;
-	double rs;
-    double si;
-    double co;
-	double y2;
-	double r;
-	double y;
-	double f;
-
-	double ONE_DIV_SQRT_PI = 0.5641895835477563;
-    double FOURTH_PI = 7.85398163397448309616e-1;
-	double x1 = 2.4048255576957727686e+00;
-	double x2 = 5.5200781102863106496e+00;
-	double x11 = 6.160e+02;
-	double x12 = -1.42444230422723137837e-03;
-	double x21 = 1.4130e+03;
-	double x22 = 5.46860286310649596604e-04;
-
-	if ( x < 0 ) {
-		x = -x;
-	}
-	if ( x == HUGE_VAL ) {
-		return 0.0;
-	}
-	if ( x == 0 ) {
-		return 1.0;
-	}
-	if ( x <= 4.0 ) {
-		y = x * x;
-		r = rational_p1q1( y );
-		f = ( x+x1 ) * ( (x - (x11/256.0)) - x12 );
-		return f * r;
-	}
-	if ( x <= 8.0 ) {
-		y = 1.0 - ( ( x*x )/64.0 );
-		r = rational_p2q2( y );
-		f = ( x+x2 ) * ( (x - (x21/256.0)) - x22 );
-		return f * r;
-	}
-	y = 8.0 / x;
-	y2 = y * y;
-	rc = rational_pcqc( y2 );
-	rs = rational_psqs( y2 );
-	f = ONE_DIV_SQRT_PI / sqrt(x);
-
-    // __sincos(x, &si, &co);
-    si = sin(x);
-    co = cos(x);
-    return f * ( ( rc * (co+si) ) - ( (y*rs) * (si-co) ) );
+    return j0(x);
 }
 
 LFORTRAN_API float _lfortran_sbesselj0( float x ) {
-    return (float)_lfortran_dbesselj0((double)x);
+    return j0(x);
+}
+
+// besselj1 --------------------------------------------------------------------
+
+LFORTRAN_API double _lfortran_dbesselj1( double x ) {
+    return j1(x);
+}
+
+LFORTRAN_API float _lfortran_sbesselj1( float x ) {
+    return j1(x);
+}
+
+// bessely0 --------------------------------------------------------------------
+
+LFORTRAN_API double _lfortran_dbessely0( double x ) {
+    return y0(x);
+}
+
+LFORTRAN_API float _lfortran_sbessely0( float x ) {
+    return y0(x);
+}
+
+// bessely1 --------------------------------------------------------------------
+
+LFORTRAN_API double _lfortran_dbessely1( double x ) {
+    return y1(x);
+}
+
+LFORTRAN_API float _lfortran_sbessely1( float x ) {
+    return y1(x);
 }
 
 // sin -------------------------------------------------------------------------
@@ -1363,6 +1196,16 @@ LFORTRAN_API double_complex_t _lfortran_zasin(double_complex_t x)
     return casin(x);
 }
 
+LFORTRAN_API float _lfortran_sasind(float x)
+{
+    return (asin(x)*180)/PI;
+}
+
+LFORTRAN_API double _lfortran_dasind(double x)
+{
+    return (asin(x)*180)/PI;
+}
+
 // acos ------------------------------------------------------------------------
 
 LFORTRAN_API float _lfortran_sacos(float x)
@@ -1385,6 +1228,16 @@ LFORTRAN_API double_complex_t _lfortran_zacos(double_complex_t x)
     return cacos(x);
 }
 
+LFORTRAN_API float _lfortran_sacosd(float x)
+{
+    return (acos(x)*180)/PI;
+}
+
+LFORTRAN_API double _lfortran_dacosd(double x)
+{
+    return (acos(x)*180)/PI;
+}
+
 // atan ------------------------------------------------------------------------
 
 LFORTRAN_API float _lfortran_satan(float x)
@@ -1564,7 +1417,9 @@ LFORTRAN_API void _lfortran_strcpy(char** x, char *y, int8_t free_target)
 {
     if (free_target) {
         if (*x) {
-            free((void *)*x);
+            // We should free `x` here, but cannot due to:
+            // https://github.com/lfortran/lfortran/issues/3787
+            //free((void *)*x);
         }
         *x = (char*) malloc((strlen(y) + 1) * sizeof(char));
         _lfortran_string_init(strlen(y) + 1, *x);
@@ -2577,7 +2432,7 @@ LFORTRAN_API void _lfortran_read_double(double *p, int32_t unit_num)
     }
 }
 
-LFORTRAN_API void _lfortran_formatted_read(int32_t unit_num, int32_t* iostat, char* fmt, int32_t no_of_args, ...)
+LFORTRAN_API void _lfortran_formatted_read(int32_t unit_num, int32_t* iostat, int32_t* chunk, char* fmt, int32_t no_of_args, ...)
 {
     if (!streql(fmt, "(a)")) {
         printf("Only (a) supported as fmt currently");
@@ -2613,11 +2468,18 @@ LFORTRAN_API void _lfortran_formatted_read(int32_t unit_num, int32_t* iostat, ch
     if (streql(*arg, "\n")) {
         *iostat = -2;
     }
-    (*arg)[strcspn(*arg, "\n")] = 0;
+    int len = strcspn(*arg, "\n");
+    *chunk = len;
+    (*arg)[len] = 0;
     va_end(args);
 }
 
 LFORTRAN_API void _lfortran_empty_read(int32_t unit_num, int32_t* iostat) {
+    if (unit_num == -1) {
+        // Read from stdin
+        return;
+    }
+
     bool unit_file_bin;
     FILE* fp = get_file_pointer_from_unit(unit_num, &unit_file_bin);
     if (!fp) {
@@ -2753,6 +2615,13 @@ LFORTRAN_API char *_lpython_get_argv(int32_t index) {
 
 // << Command line arguments << ------------------------------------------------
 
+// Initial setup
+LFORTRAN_API void _lpython_call_initial_functions(int32_t argc_1, char *argv_1[]) {
+    _lpython_set_argv(argc_1, argv_1);
+    _lfortran_init_random_clock();
+}
+// << Initial setup << ---------------------------------------------------------
+
 // >> Runtime Stacktrace >> ----------------------------------------------------
 #ifdef HAVE_RUNTIME_STACKTRACE
 #ifdef HAVE_LFORTRAN_UNWIND
